package io.renren.modules.admin.service;

import java.util.Map;
import com.baomidou.mybatisplus.extension.service.IService;
import io.renren.common.utils.PageUtils;

import io.renren.modules.admin.entity.MemberEntity;


/**
 * 
 *
 * @author huwhois
 * @email huwhois@163.com
 * @date 2020-12-09 14:33:56
 */
public interface MemberService extends IService<MemberEntity> {

    PageUtils queryPage(Map<String, Object> params);
    
    /**
     * 修改备注
     * @param id
     * @param remark
     * @return
     */
    int updateRemark(Long id, String remark);
    
    int updateFlag(Long id, Integer flag);

    /**
     * 通过标识查找会员信息
     * @param symbol
     * @return
     */
    MemberEntity getBySymbol(String symbol);

    boolean saveMember(MemberEntity member);

    boolean updateMember(MemberEntity member);

    PageUtils queryInvitationPage(Map<String, Object> params);
    
    MemberEntity saveMember(String truename, String phone, String organization, String position);

}

